
#include "cerberus.h"
#include "entities.h"


/*!
 * Initialising list variables.
 */
cListItem::cListItem() : mPreviousListItem( NULL ), mNextListItem( NULL )
{
}


/*!
 * Clean up list item.
 */
cListItem::~cListItem()
{
}


/*!
 * Setting up variables.
 */
cEntity::cEntity( cWorld* world, IrrlichtDevice* device ) : mWorld( world ), mDevice( device )
{
	mNode=NULL;
}


/*!
 * Remove node from scene manager
 */
cEntity::~cEntity( void )
{
	if( mNode )
		mNode->remove();
}


/*!
 * Setting up variables.
 */
cElement::cElement( cWorld* world, IrrlichtDevice* device ) : mWorld( world ), mDevice( device )
{
  mIndexI=0; mIndexJ=0; mLevel=0;
 	mNode=NULL;
  mFlat=false;
  
	mTree=NULL;
	mBoulder=NULL;
	mRobot=NULL;

  mLogger = mDevice->getLogger();
}


/*!
 * Free entities on this element and remove node.
 */
cElement::~cElement( void )
{
	if( mTree )
		delete mTree;

	if( mBoulder ) 
		delete mBoulder;

	if( mRobot )
		delete mRobot;
		
	mNode->remove();
}


/*!
 * 
 */
bool cElement::createEntity( eEntity entity )
{
  if( !isFree() )
    return false;
  
  switch( entity ) {
  case eTree:
    {
      mTree = new cEntity( mWorld, mDevice );
      scene::IAnimatedMeshSceneNode* Node=mDevice->getSceneManager()->addAnimatedMeshSceneNode( mWorld->getTreeMesh(), mWorld->getNode() );
      if( Node ) {
        Node->setMaterialFlag( video::EMF_LIGHTING, false );
        Node->setPosition( core::vector3df(mIndexI+0.5, mLevel*sqrt(2), mIndexJ+0.5) );
        Node->setRotation( core::vector3df(0.0, mWorld->getRandomNumber()*360.0, 0.0) );
        mTree->setNode( Node );
      } else {
        fprintf( stdout, "couldn't allocate tree" );
      }
    }
    break;
  case eBoulder:
    {
      mBoulder = new cEntity( mWorld, mDevice );
      scene::IAnimatedMeshSceneNode* Node=mDevice->getSceneManager()->addAnimatedMeshSceneNode( mWorld->getBoulderMesh(), mWorld->getNode() );
      if( Node ) {
        Node->setMaterialFlag( video::EMF_LIGHTING, false );
        Node->setPosition( core::vector3df(mIndexI+0.5, mLevel*sqrt(2), mIndexJ+0.5) );
        Node->setRotation( core::vector3df(0.0, mWorld->getRandomNumber()*360.0, 0.0) );
        mBoulder->setNode( Node );
      } else {
        fprintf( stdout, "couldn't allocate tree" );
      }
    }
    break;
  case eRobot:
    {
      mRobot = new cEntity( mWorld, mDevice );
      scene::IAnimatedMeshSceneNode* Node=mDevice->getSceneManager()->addAnimatedMeshSceneNode( mWorld->getRobotMesh(), mWorld->getNode() );
      if( !Node ) {
        fprintf( stdout, "couldn't allocate robot" );
      }

      Node->setMaterialFlag( video::EMF_LIGHTING, false );
      Node->setPosition( core::vector3df(mIndexI+0.5, mLevel*sqrt(2), mIndexJ+0.5) );
      
      core::vector3df diffVector=mWorld->getSoul()->getPosition()-Node->getPosition();
      Node->setRotation( core::vector3df(0.0, 180.0-atan2( diffVector.Z, diffVector.X )*180.0/core::PI, 0.0) );
      mRobot->setNode( Node );
    }
    break;
  default:
    return false;
  }
	
	return true;
}


/*!
 * 
 */
int cElement::absorbEntity( void )
{
	int energy=0;
	
	if( mTree ) {
		delete mTree;
		mTree=NULL;
		energy=eTreeNrg;
	} else if( mBoulder ) {
		delete mBoulder;
		mBoulder=NULL;
		energy=eBoulderNrg;
	}	else if( mRobot ) {
		delete mRobot;
		mRobot=NULL;
		energy=eRobotNrg;
	}
	
	return energy;
}


/*!
 * 
 */
bool cElement::isFree( void )
{
	if( mFlat && mTree==NULL && mRobot==NULL
	    && mBoulder==NULL )
		return true;
	
	return false;
}

/*!
 * 
 */
eEntity cElement::getEntity( void )
{
	if( mTree )
		return eTree;
	if( mBoulder )
		return eBoulder;
	if( mRobot )
		return eRobot;
		
	return eNil;
}


/*!
 * 
 */
void cElement::rotateRobot( void )
{
  if( mRobot ) {
    scene::IAnimatedMeshSceneNode* Node=mRobot->getNode();
    core::vector3df diffVector=mWorld->getSoul()->getPosition()-Node->getPosition();
    Node->setRotation( core::vector3df(0.0, 180-atan2( diffVector.Z, diffVector.X )*180.0/core::PI, 0.0) );
  }
}

/*!
 * Initialize soul and camera.
 *
 * @param camera : pointer to the camera node, of which the soul takes care of
 */
cSoul::cSoul( scene::ICameraSceneNode* camera ) : mCamera( camera ), mTheta( core::PI/2.0 ), 
        mThetaMin( 2.0f/180.0f*core::PI ), mThetaMax( 178.0f/180.0f*core::PI )
{
	// camera settings
  mRotateSpeed=1.0f;
  mCamera->setNearValue( 0.1f );
  mCamera->setFarValue( 12000.0f );
  
  // internal settings
  mEnergy = 7;
  
  // soul always looks to the middle of the landscape
  mPhi=atan2( (double)(16-mIndexJ), (double)(16-mIndexI) );
  
	// initialize View
  updateView();
}


/*!
 * Update view (direction where we looking at).
 */
void cSoul::updateView( void )
{
	mCamera->setPosition( core::vector3df( mIndexI+0.5, mHeight, mIndexJ+0.5 ) );
	mCamera->setTarget( mCamera->getPosition() +
		core::vector3df(cos(mPhi)*sin(mTheta), cos(mTheta), sin(mPhi)*sin(mTheta)) );            
	mCamera->setUpVector( core::vector3df(-cos(mPhi)*cos(mTheta), sin(mTheta), -sin(mPhi)*cos(mTheta)) );            
}


/*!
 * 
 */
void cSoul::setTarget( core::vector3df target )
{
	core::vector3df diffVector=target-getPosition();

	mPhi=atan2( diffVector.Z, diffVector.X );
	mTheta=atan2( sqrt(diffVector.X*diffVector.X+diffVector.Z*diffVector.Z), diffVector.Y );
}


/*!
 * Change view (direction where we looking at).
 */
void cSoul::changeView( f32 px, f32 py )
{
	// determine new rotation angles
	mPhi -= px*mRotateSpeed;
	mTheta += py*mRotateSpeed;
	if( mTheta<mThetaMin ) mTheta=mThetaMin;
	if( mTheta>mThetaMax ) mTheta=mThetaMax;
	if( mPhi<0.0 ) 				mPhi+=2.0f*core::PI;
	if( mPhi>2*core::PI )  mPhi-=2.0f*core::PI;
	
	updateView();
}


/*!
 * Quick turn around.
 */
void cSoul::turnAround()
{
	// turn around 180 degrees
	mPhi += core::PI;
	if( mPhi>2*core::PI )  mPhi-=2.0f*core::PI;
	mTheta = core::PI/2;
	
	updateView();
}


/*!
 * Substract an amount of energy from the souls energy.
 * If the energy of the soul becomes 0 or lower return
 * false - the soul dies of exhaustion.
 *
 * @param sub : amount of energy to substract
 * @return false if energy exhausted, true otherwise
 */
bool cSoul::substractFromEnergy( int sub )
{ 
  mEnergy -= sub;

  if( mEnergy<=0 )
    return false;
  
  return true;
}
