
#ifndef __ENTITIES_H
#define __ENTITIES_H

// include irrlicht headers
#include "irrlicht.h"

// forward declarations
class cWorld;


enum eEntity { eNil=0, eTree=1, eBoulder, eRobot, eMeanie, eSentinel };
enum eEnergy { eNilNrg=0, eTreeNrg=1, eBoulderNrg=2, eRobotNrg=3 };

class cListItem {
public:
	cListItem();
	~cListItem();
	
private:	
	cListItem* mPreviousListItem;
	cListItem* mNextListItem;
};

class cEntity : public cListItem {
public:
  cEntity( cWorld* world, IrrlichtDevice* device );
	~cEntity();
	
	void setNode( scene::IAnimatedMeshSceneNode* Node )
		{ mNode=Node; }
	scene::IAnimatedMeshSceneNode* getNode()
		{ return mNode; }
  
private:
  scene::IAnimatedMeshSceneNode* mNode;
  cWorld* mWorld;
  IrrlichtDevice* mDevice;
};


class cSoul {
public:
  // constructor
  cSoul( scene::ICameraSceneNode* camera );

  void changeView( f32 px, f32 py );  
  void turnAround();  
  void updateView();  

  void setIndex( int i, int j )
    { mIndexI=i; mIndexJ=j; }
  void setLevel( int level )
    { mLevel=level; }
  int getLevel()
    { return mLevel;}
  void setHeight( f32 height )
    { mHeight=height; }
  int getIndexI()
    { return mIndexI;}
  int getIndexJ()
    { return mIndexJ;}
  f32 getHeight()
    { return mHeight;}
  core::vector3df getPosition()
    { return core::vector3df( mIndexI+0.5, mHeight, mIndexJ+0.5 ); }
	void setTarget( core::vector3df target );

  // manage energy of soul
  int getEnergy()
    { return mEnergy; }
  void addToEnergy( int add )
    { mEnergy += add; }
  bool substractFromEnergy( int sub );
    
private:
  scene::ICameraSceneNode* mCamera;
  f32 mTheta, mPhi;
  const f32 mThetaMin, mThetaMax;    
  f32 mRotateSpeed;
  int mEnergy;
  int mIndexI, mIndexJ, mLevel;
  f32 mHeight;
};


class cElement {
public:
	cElement( cWorld* world, IrrlichtDevice* device );
	~cElement();
    
  void setSoul( cSoul* soul )
    { mSoul=soul; mRobot->getNode()->setVisible( mSoul ? false : true ); }    
  cSoul* getSoul()
    { return mSoul; }    
    
  void setIndex( int i, int j )
    { mIndexI=i; mIndexJ=j; }
  void setLevel( int level )
    { mLevel=level;}
  int getIndexI()
    { return mIndexI;}
  int getIndexJ()
    { return mIndexJ;}
  int getLevel()
    { return mLevel;}
  core::vector3df getPosition()
    { return core::vector3df( mIndexI+0.5, mLevel*sqrt(2.0), mIndexJ+0.5 ); }
  
  void rotateRobot();
    
  void setNode( scene::IMeshSceneNode* Node )
    { mNode=Node; }
  scene::IMeshSceneNode* GetNode()
    { return mNode; };
    
  void setFlat( bool flat )
    { mFlat=flat; }
  bool getFlat()
    { return mFlat; }
		
	bool isFree();
	eEntity getEntity();

  bool createEntity( eEntity entity );
	int absorbEntity();
    
private:
  cWorld* mWorld;
  IrrlichtDevice* mDevice;
  ILogger *mLogger;
  scene::IMeshSceneNode* mNode;
	int mIndexI, mIndexJ, mLevel;
  bool mFlat;
	
	cEntity* mTree;
	cEntity* mBoulder;
	cEntity* mRobot;
  cSoul*   mSoul;
};

#endif // __ENTITIES_H
